@@ -36,10 +36,15 @@ class BaseAlgo:
     """
 
     def __init__(self):
-        self.trader = None
-        self.interval = None
-        self.aggregations = None
-        self.watchlist = []
+        self.interval = None  # Interval to run the algorithm
+        self.aggregations = None  # Intervals to aggregate data
+        self.watchlist = []  # List of assets this algorithm tracks
+
+    def init(self, stats, func, account):
+        self.stats = stats
+        self.func = func
+        self.account = account
+        self.positions = account.positions
 
     def config(self):
         """
@@ -109,7 +114,7 @@ class BaseAlgo:
             quantity = self.get_asset_max_quantity(symbol)
 
         debugger.debug(f"Algo BUY: {symbol}, {quantity}")
-        return self.trader.buy(symbol, quantity, in_force, extended)
+        return self.func.buy(symbol, quantity, in_force, extended)
 
     def sell(
         self,
@@ -140,7 +145,7 @@ class BaseAlgo:
             quantity = self.get_asset_quantity(symbol)
 
         debugger.debug(f"Algo SELL: {symbol}, {quantity}")
-        return self.trader.sell(symbol, quantity, in_force, extended)
+        return self.func.sell(symbol, quantity, in_force, extended)
 
     def sell_all_options(self, symbol: str = None, in_force: str = "gtc"):
         """Sells all options of a stock
@@ -154,78 +159,16 @@ class BaseAlgo:
         if symbol is None:
             symbol = self.watchlist[0]
 
-        symbols = [
-            s["symbol"]
-            for s in self.get_account_option_positions()
-            if s["base_symbol"] == symbol
-        ]
+        symbols = filter(lambda x: x.base_symbol == symbol, self.positions.option)
+
         ret = []
         for s in symbols:
             debugger.debug(f"Algo SELL OPTION: {s}")
-            quantity = self.get_asset_quantity(s)
-            ret.append(self.trader.sell(s, quantity, in_force, True))
+            quantity = self.get_asset_quantity(s.symbol)
+            ret.append(self.func.sell(s.symbol, quantity, in_force, True))
 
         return ret
 
-    # def buy_option(self, symbol: str, quantity: int = None, in_force: str = "gtc"):
-    #     """Buys the specified option.
-
-    #     When called, a limit buy order is placed with a limit
-    #     price 5% higher than the current price.
-
-    #     :param str symbol:    Symbol of the asset to buy, in {OCC} format.
-    #     :param float? quantity:  Quantity of asset to buy. defaults to buys as many as possible
-    #     :param str? in_force:  Duration the order is in force. '{gtc}' or '{gtd}'. defaults to 'gtc'
-    #     :returns: A dictionary with the following keys:
-
-    #         - type: 'OPTION'
-    #         - id: ID of order
-    #         - symbol: symbol of asset
-
-    #     :raises Exception: There is an error in the order process.
-    #     """
-    #     if quantity is None:
-    #         quantity = self.get_asset_max_quantity(symbol)
-    #     return self.trader.buy_option(symbol, quantity, in_force)
-
-    # def sell_option(
-    #     self, symbol: str = None, quantity: int = None, in_force: str = "gtc"
-    # ):
-    #     """Sells the specified option.
-
-    #     When called, a limit sell order is placed with a limit
-    #     price 5% lower than the current price.
-
-    #     If the option symbol is specified, it will sell that option. If it is not, then the
-    #     method will select the first stock symbol in the watchlist, and sell all options
-    #     related to that stock.
-
-    #     :param str? symbol: Symbol of the asset to sell, in {OCC} format. defaults to sell all options for the first stock in watchlist
-    #     :param float? quantity:  Quantity of asset to sell. defaults to sells all
-    #     :param str? in_force:  Duration the order is in force. '{gtc}' or '{gtd}'. defaults to 'gtc'
-    #     :returns: A dictionary with the following keys:
-
-    #         - type: 'OPTION'
-    #         - id: ID of order
-    #         - symbol: symbol of asset
-
-    #     :raises Exception: There is an error in the order process.
-    #     """
-    #     if symbol is None:
-    #         symbol = self.watchlist[0]
-    #         symbols = [
-    #             s["symbol"]
-    #             for s in self.get_account_option_positions()
-    #             if s["base_symbol"] == symbol
-    #         ]
-    #     else:
-    #         symbols = [symbol]
-    #     for s in symbols:
-    #         debugger.debug(f"Algo SELL OPTION: {s}")
-    #         if quantity is None:
-    #             quantity = self.get_asset_quantity(s)
-    #         return self.trader.sell_option(s, quantity, in_force)
-
     def filter_option_chain(
         self,
         symbol=None,
@@ -248,8 +191,8 @@ class BaseAlgo:
         """
         if symbol is None:
             symbol = self.watchlist[0]
-        lower_exp = convert_input_to_datetime(lower_exp, self.trader.timezone)
-        upper_exp = convert_input_to_datetime(upper_exp, self.trader.timezone)
+        lower_exp = convert_input_to_datetime(lower_exp, self.stats.timezone)
+        upper_exp = convert_input_to_datetime(upper_exp, self.stats.timezone)
 
         exp_dates = self.get_option_chain_info(symbol)["exp_dates"]
         if lower_exp is not None:
@@ -283,9 +226,9 @@ class BaseAlgo:
             - exp_dates: List of expiration dates, in the fomrat "YYYY-MM-DD"
             - multiplier: Multiplier of the option, usually 100
         """
-        if symbol == None:
+        if symbol is None:
             symbol = self.watchlist[0]
-        return self.trader.fetch_chain_info(symbol)
+        return self.func.fetch_chain_info(symbol)
 
     def get_option_chain(self, symbol: str, date):
         """Returns the option chain for the specified symbol and expiration date.
@@ -303,9 +246,9 @@ class BaseAlgo:
         """
         if symbol is None:
             symbol = self.watchlist[0]
-        date = convert_input_to_datetime(date, self.trader.timezone)
+        date = convert_input_to_datetime(date, self.stats.timezone)
         print(f"Date: {date}\n")
-        return self.trader.fetch_chain_data(symbol, date)
+        return self.func.fetch_chain_data(symbol, date)
 
     def get_option_market_data(self, symbol: str):
         """Retrieves data of specified option.
@@ -320,27 +263,21 @@ class BaseAlgo:
         """
         if symbol is None:
             symbol = self.watchlist[0]
-        return self.trader.fetch_option_market_data(symbol)
+        return self.func.fetch_option_market_data(symbol)
 
     # ------------------ Technical Indicators -------------------
 
     def _default_param(self, symbol, interval, ref, prices):
         if symbol is None:
             symbol = self.watchlist[0]
-        if self.trader is None:
-            if interval == None:
-                interval = Interval.MIN_5
-            else:
-                interval = interval_string_to_enum(interval)
-            if prices is None:
-                raise Exception(f"No prices found for symbol {symbol}")
+
+        if interval is None:
+            interval = self.stats.watchlist_cfg[symbol]["interval"]
         else:
-            if interval is None:
-                interval = self.trader.interval[symbol]["interval"]
-            else:
-                interval = interval_string_to_enum(interval)
-            if prices == None:
-                prices = self.trader.storage.load(symbol, interval)[symbol][ref]
+            interval = interval_string_to_enum(interval)
+
+        if prices is None:
+            prices = self.func.load(symbol, interval)[symbol][ref]
 
         return symbol, interval, ref, prices
 
@@ -523,7 +460,7 @@ class BaseAlgo:
         if symbol is None:
             symbol = self.watchlist[0]
 
-        return self.trader.get_asset_quantity(symbol, exclude_pending_sell=True)
+        return self.func.get_asset_quantity(symbol, exclude_pending_sell=True)
 
     def get_asset_cost(self, symbol: str = None) -> float:
         """Returns the average cost of a specified asset.
@@ -535,14 +472,14 @@ class BaseAlgo:
         if symbol is None:
             symbol = self.watchlist[0]
         if len(symbol) <= 6:
-            search = self.trader.stock_positions + self.trader.crypto_positions
+            search = self.positions.stock_crypto
             for p in search:
-                if p["symbol"] == symbol:
-                    return p["avg_price"]
+                if p.symbol == symbol:
+                    return p.avg_price
         else:
-            for p in self.trader.option_positions:
+            for p in self.positions.option:
                 if p["occ_symbol"].replace(" ", "") == symbol.replace(" ", ""):
-                    return p["avg_price"]
+                    return p.avg_price
 
         raise Exception(f"{symbol} is not currently owned")
 
@@ -556,11 +493,11 @@ class BaseAlgo:
         if symbol is None:
             symbol = self.watchlist[0]
         if len(symbol) <= 6:
-            return self.trader.storage.load(symbol, self.interval)[symbol]["close"][-1]
-        for p in self.trader.option_positions:
+            return self.func.load(symbol, self.interval)[symbol]["close"][-1]
+        for p in self.positions.option:
             if p["occ_symbol"] == symbol:
                 return p["current_price"] * p["multiplier"]
-        return self.get_option_market_data(symbol)["price"] * 100
+        return self.func.fetch_option_market_data(symbol)["price"] * 100
 
     def get_asset_price_list(
         self, symbol: str = None, interval: str = None, ref: str = "close"
@@ -581,7 +518,7 @@ class BaseAlgo:
         else:
             interval = interval_string_to_enum(interval)
         if len(symbol) <= 6:
-            return list(self.trader.storage.load(symbol, interval)[symbol][ref])
+            return list(self.func.load(symbol, interval)[symbol][ref])
         debugger.warning("Price list not available for options")
         return None
 
@@ -608,9 +545,9 @@ class BaseAlgo:
         if interval is None:
             interval = self.interval
         if len(symbol) <= 6:
-            df = self.trader.storage.load(symbol, interval).iloc[[-1]][symbol]
-            print(self.trader.storage.load(symbol, interval))
-            return pandas_timestamp_to_local(df, self.trader.timezone)
+            df = self.func.load(symbol, interval).iloc[[-1]][symbol]
+            print(self.func.load(symbol, interval))
+            return pandas_timestamp_to_local(df, self.stats.timezone)
         debugger.warning("Candles not available for options")
         return None
 
@@ -638,8 +575,8 @@ class BaseAlgo:
             symbol = self.watchlist[0]
         if interval is None:
             interval = self.interval
-        df = self.trader.storage.load(symbol, interval)[symbol]
-        return pandas_timestamp_to_local(df, self.trader.timezone)
+        df = self.func.load(symbol, interval)[symbol]
+        return pandas_timestamp_to_local(df, self.stats.timezone)
 
     def get_asset_returns(self, symbol=None) -> float:
         """Returns the return of a specified asset.
@@ -682,14 +619,14 @@ class BaseAlgo:
 
         :returns: The current buying power as a float.
         """
-        return self.trader.account["buying_power"]
+        return self.account.buying_power
 
     def get_account_equity(self) -> float:
         """Returns the current equity.
 
         :returns: The current equity as a float.
         """
-        return self.trader.account["equity"]
+        return self.account.equity
 
     def get_account_stock_positions(self) -> List:
         """Returns the current stock positions.
@@ -699,7 +636,7 @@ class BaseAlgo:
             - quantity
             - avg_price
         """
-        return self.trader.stock_positions
+        return self.positions.stock
 
     def get_account_crypto_positions(self) -> List:
         """Returns the current crypto positions.
@@ -709,7 +646,7 @@ class BaseAlgo:
             - quantity
             - avg_price
         """
-        return self.trader.crypto_positions
+        return self.positions.crypto
 
     def get_account_option_positions(self) -> List:
         """Returns the current option positions.
@@ -721,12 +658,12 @@ class BaseAlgo:
         """
         return [
             {
-                "symbol": p["symbol"],
-                "base_symbol": p["base_symbol"],
-                "quantity": p["quantity"],
-                "avg_price": p["avg_price"],
+                "symbol": p.symbol,
+                "base_symbol": p.base_symbol,
+                "quantity": p.quantity,
+                "avg_price": p.avg_price,
             }
-            for p in self.trader.option_positions
+            for p in self.positions.option
         ]
 
     def get_watchlist(self) -> List:
@@ -770,9 +707,7 @@ class BaseAlgo:
 
         :returns: The current date and time as a datetime object
         """
-        return datetime_utc_to_local(
-            self.trader.streamer.timestamp, self.trader.timezone
-        )
+        return datetime_utc_to_local(self.stats.timestamp, self.stats.timezone)
 
     def get_option_position_quantity(self, symbol: str = None) -> bool:
         """Returns the number of types of options held for a stock.
@@ -782,7 +717,7 @@ class BaseAlgo:
         """
         if symbol is None:
             symbol = self.watchlist[0]
-        pos = [p for p in self.trader.option_positions if p["symbol"] == symbol]
+        pos = [p for p in self.positions.option if p.symbol == symbol]
         return len(pos)
 
     def is_day_trade(self, action, symbol=None) -> bool:
